package javax.swing.text.html;

import java.awt.HeadlessException;
import java.awt.Image;
import java.awt.Toolkit;
import javax.swing.ImageIcon;
import javax.swing.text.View;
import java.io.ObjectInputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import javax.swing.SizeRequirements;
import java.util.Enumeration;
import java.util.Vector;
import java.net.MalformedURLException;
import java.net.URL;
import javax.swing.text.Element;
import javax.swing.text.SimpleAttributeSet;
import java.awt.Color;
import java.awt.Font;
import javax.swing.text.AttributeSet;
import javax.swing.text.StyleContext;
import javax.swing.text.StyleConstants;
import javax.swing.text.MutableAttributeSet;
import java.util.Hashtable;
import java.io.Serializable;

public class CSS implements Serializable
{
    private static final Hashtable<String, CSS.Attribute> attributeMap;
    private static final Hashtable<String, CSS.Value> valueMap;
    private static final Hashtable<HTML.Attribute, CSS.Attribute[]> htmlAttrToCssAttrMap;
    private static final Hashtable<Object, CSS.Attribute> styleConstantToCssMap;
    private static final Hashtable<String, CSS.Value> htmlValueToCssValueMap;
    private static final Hashtable<String, CSS.Value> cssValueToInternalValueMap;
    private transient Hashtable<Object, Object> valueConvertor;
    private int baseFontSize;
    private transient StyleSheet styleSheet;
    static int baseFontSizeIndex;
    
    public CSS() {
        this.styleSheet = null;
        this.baseFontSize = CSS.baseFontSizeIndex + 1;
        (this.valueConvertor = new Hashtable()).put(CSS.Attribute.FONT_SIZE, new CSS.FontSize());
        this.valueConvertor.put(CSS.Attribute.FONT_FAMILY, new CSS.FontFamily());
        this.valueConvertor.put(CSS.Attribute.FONT_WEIGHT, new CSS.FontWeight());
        CSS.BorderStyle borderStyle = new CSS.BorderStyle();
        this.valueConvertor.put(CSS.Attribute.BORDER_TOP_STYLE, borderStyle);
        this.valueConvertor.put(CSS.Attribute.BORDER_RIGHT_STYLE, borderStyle);
        this.valueConvertor.put(CSS.Attribute.BORDER_BOTTOM_STYLE, borderStyle);
        this.valueConvertor.put(CSS.Attribute.BORDER_LEFT_STYLE, borderStyle);
        CSS.ColorValue colorValue = new CSS.ColorValue();
        this.valueConvertor.put(CSS.Attribute.COLOR, colorValue);
        this.valueConvertor.put(CSS.Attribute.BACKGROUND_COLOR, colorValue);
        this.valueConvertor.put(CSS.Attribute.BORDER_TOP_COLOR, colorValue);
        this.valueConvertor.put(CSS.Attribute.BORDER_RIGHT_COLOR, colorValue);
        this.valueConvertor.put(CSS.Attribute.BORDER_BOTTOM_COLOR, colorValue);
        this.valueConvertor.put(CSS.Attribute.BORDER_LEFT_COLOR, colorValue);
        CSS.LengthValue value = new CSS.LengthValue();
        this.valueConvertor.put(CSS.Attribute.MARGIN_TOP, value);
        this.valueConvertor.put(CSS.Attribute.MARGIN_BOTTOM, value);
        this.valueConvertor.put(CSS.Attribute.MARGIN_LEFT, value);
        this.valueConvertor.put(CSS.Attribute.MARGIN_LEFT_LTR, value);
        this.valueConvertor.put(CSS.Attribute.MARGIN_LEFT_RTL, value);
        this.valueConvertor.put(CSS.Attribute.MARGIN_RIGHT, value);
        this.valueConvertor.put(CSS.Attribute.MARGIN_RIGHT_LTR, value);
        this.valueConvertor.put(CSS.Attribute.MARGIN_RIGHT_RTL, value);
        this.valueConvertor.put(CSS.Attribute.PADDING_TOP, value);
        this.valueConvertor.put(CSS.Attribute.PADDING_BOTTOM, value);
        this.valueConvertor.put(CSS.Attribute.PADDING_LEFT, value);
        this.valueConvertor.put(CSS.Attribute.PADDING_RIGHT, value);
        CSS.BorderWidthValue borderWidthValue = new CSS.BorderWidthValue(null, 0);
        this.valueConvertor.put(CSS.Attribute.BORDER_TOP_WIDTH, borderWidthValue);
        this.valueConvertor.put(CSS.Attribute.BORDER_BOTTOM_WIDTH, borderWidthValue);
        this.valueConvertor.put(CSS.Attribute.BORDER_LEFT_WIDTH, borderWidthValue);
        this.valueConvertor.put(CSS.Attribute.BORDER_RIGHT_WIDTH, borderWidthValue);
        this.valueConvertor.put(CSS.Attribute.TEXT_INDENT, new CSS.LengthValue(true));
        this.valueConvertor.put(CSS.Attribute.WIDTH, value);
        this.valueConvertor.put(CSS.Attribute.HEIGHT, value);
        this.valueConvertor.put(CSS.Attribute.BORDER_SPACING, value);
        CSS.StringValue stringValue = new CSS.StringValue();
        this.valueConvertor.put(CSS.Attribute.FONT_STYLE, stringValue);
        this.valueConvertor.put(CSS.Attribute.TEXT_DECORATION, stringValue);
        this.valueConvertor.put(CSS.Attribute.TEXT_ALIGN, stringValue);
        this.valueConvertor.put(CSS.Attribute.VERTICAL_ALIGN, stringValue);
        CSS.CssValueMapper value2 = new CSS.CssValueMapper();
        this.valueConvertor.put(CSS.Attribute.LIST_STYLE_TYPE, value2);
        this.valueConvertor.put(CSS.Attribute.BACKGROUND_IMAGE, new CSS.BackgroundImage());
        this.valueConvertor.put(CSS.Attribute.BACKGROUND_POSITION, new CSS.BackgroundPosition());
        this.valueConvertor.put(CSS.Attribute.BACKGROUND_REPEAT, value2);
        this.valueConvertor.put(CSS.Attribute.BACKGROUND_ATTACHMENT, value2);
        CSS.CssValue value3 = new CSS.CssValue();
        for (int length = CSS.Attribute.allAttributes.length, i = 0; i < length; ++i) {
            CSS.Attribute attribute = CSS.Attribute.allAttributes[i];
            if (this.valueConvertor.get(attribute) == null) {
                this.valueConvertor.put(attribute, value3);
            }
        }
    }
    
    void setBaseFontSize(int baseFontSize) {
        if (baseFontSize < 1) {
            this.baseFontSize = 0;
        }
        else if (baseFontSize > 7) {
            this.baseFontSize = 7;
        }
        else {
            this.baseFontSize = baseFontSize;
        }
    }
    
    void setBaseFontSize(String s) {
        if (s != null) {
            if (s.startsWith("+")) {
                this.setBaseFontSize(this.baseFontSize + (int)Integer.valueOf(s.substring(1)));
            }
            else if (s.startsWith("-")) {
                this.setBaseFontSize(this.baseFontSize + -(int)Integer.valueOf(s.substring(1)));
            }
            else {
                this.setBaseFontSize((int)Integer.valueOf(s));
            }
        }
    }
    
    int getBaseFontSize() {
        return this.baseFontSize;
    }
    
    void addInternalCSSValue(MutableAttributeSet set, CSS.Attribute attribute, String s) {
        if (attribute == CSS.Attribute.FONT) {
            CSS.ShorthandFontParser.parseShorthandFont(this, s, set);
        }
        else if (attribute == CSS.Attribute.BACKGROUND) {
            CSS.ShorthandBackgroundParser.parseShorthandBackground(this, s, set);
        }
        else if (attribute == CSS.Attribute.MARGIN) {
            CSS.ShorthandMarginParser.parseShorthandMargin(this, s, set, CSS.Attribute.ALL_MARGINS);
        }
        else if (attribute == CSS.Attribute.PADDING) {
            CSS.ShorthandMarginParser.parseShorthandMargin(this, s, set, CSS.Attribute.ALL_PADDING);
        }
        else if (attribute == CSS.Attribute.BORDER_WIDTH) {
            CSS.ShorthandMarginParser.parseShorthandMargin(this, s, set, CSS.Attribute.ALL_BORDER_WIDTHS);
        }
        else if (attribute == CSS.Attribute.BORDER_COLOR) {
            CSS.ShorthandMarginParser.parseShorthandMargin(this, s, set, CSS.Attribute.ALL_BORDER_COLORS);
        }
        else if (attribute == CSS.Attribute.BORDER_STYLE) {
            CSS.ShorthandMarginParser.parseShorthandMargin(this, s, set, CSS.Attribute.ALL_BORDER_STYLES);
        }
        else if (attribute == CSS.Attribute.BORDER || attribute == CSS.Attribute.BORDER_TOP || attribute == CSS.Attribute.BORDER_RIGHT || attribute == CSS.Attribute.BORDER_BOTTOM || attribute == CSS.Attribute.BORDER_LEFT) {
            CSS.ShorthandBorderParser.parseShorthandBorder(set, attribute, s);
        }
        else {
            Object internalCSSValue = this.getInternalCSSValue(attribute, s);
            if (internalCSSValue != null) {
                set.addAttribute(attribute, internalCSSValue);
            }
        }
    }
    
    Object getInternalCSSValue(CSS.Attribute key, String s) {
        CSS.CssValue cssValue = (CSS.CssValue)this.valueConvertor.get(key);
        Object cssValue2 = cssValue.parseCssValue(s);
        return (cssValue2 != null) ? cssValue2 : cssValue.parseCssValue(key.getDefaultValue());
    }
    
    CSS.Attribute styleConstantsKeyToCSSKey(StyleConstants key) {
        return (CSS.Attribute)CSS.styleConstantToCssMap.get(key);
    }
    
    Object styleConstantsValueToCSSValue(StyleConstants styleConstants, Object o) {
        CSS.Attribute styleConstantsKeyToCSSKey = this.styleConstantsKeyToCSSKey(styleConstants);
        if (styleConstantsKeyToCSSKey != null) {
            return ((CSS.CssValue)this.valueConvertor.get(styleConstantsKeyToCSSKey)).fromStyleConstants(styleConstants, o);
        }
        return null;
    }
    
    Object cssValueToStyleConstantsValue(StyleConstants styleConstants, Object o) {
        if ((o instanceof CSS.CssValue)) {
            return ((CSS.CssValue)o).toStyleConstants(styleConstants, null);
        }
        return null;
    }
    
    Font getFont(StyleContext styleContext, AttributeSet set, int n, StyleSheet styleSheet) {
        styleSheet = this.getStyleSheet(styleSheet);
        int fontSize = getFontSize(set, n, styleSheet);
        CSS.StringValue stringValue = (CSS.StringValue)set.getAttribute(CSS.Attribute.VERTICAL_ALIGN);
        if (stringValue != null) {
            String string = stringValue.toString();
            if (string.indexOf("sup") >= 0 || string.indexOf("sub") >= 0) {
                fontSize -= 2;
            }
        }
        CSS.FontFamily fontFamily = (CSS.FontFamily)set.getAttribute(CSS.Attribute.FONT_FAMILY);
        String family = (fontFamily != null) ? fontFamily.getValue() : "SansSerif";
        int n2 = 0;
        CSS.FontWeight fontWeight = (CSS.FontWeight)set.getAttribute(CSS.Attribute.FONT_WEIGHT);
        if (fontWeight != null && fontWeight.getValue() > 400) {
            n2 |= 0x1;
        }
        Object attribute = set.getAttribute(CSS.Attribute.FONT_STYLE);
        if (attribute != null && attribute.toString().indexOf("italic") >= 0) {
            n2 |= 0x2;
        }
        if (family.equalsIgnoreCase("monospace")) {
            family = "Monospaced";
        }
        Font font = styleContext.getFont(family, n2, fontSize);
        if (font == null || (font.getFamily().equals("Dialog") && !family.equalsIgnoreCase("Dialog"))) {
            font = styleContext.getFont("SansSerif", n2, fontSize);
            return font;
        }
        return font;
    }
    
    static int getFontSize(AttributeSet set, int n, StyleSheet styleSheet) {
        CSS.FontSize fontSize = (CSS.FontSize)set.getAttribute(CSS.Attribute.FONT_SIZE);
        return (fontSize != null) ? fontSize.getValue(set, styleSheet) : n;
    }
    
    Color getColor(AttributeSet set, CSS.Attribute attribute) {
        CSS.ColorValue colorValue = (CSS.ColorValue)set.getAttribute(attribute);
        if (colorValue != null) {
            return colorValue.getValue();
        }
        return null;
    }
    
    float getPointSize(String s, StyleSheet styleSheet) {
        styleSheet = this.getStyleSheet(styleSheet);
        if (s != null) {
            if (s.startsWith("+")) {
                return this.getPointSize(this.baseFontSize + (int)Integer.valueOf(s.substring(1)), styleSheet);
            }
            if (s.startsWith("-")) {
                return this.getPointSize(this.baseFontSize + -(int)Integer.valueOf(s.substring(1)), styleSheet);
            }
            return this.getPointSize((int)Integer.valueOf(s), styleSheet);
        }
        else {
            return 0.0f;
        }
    }
    
    float getLength(AttributeSet set, CSS.Attribute attribute, StyleSheet styleSheet) {
        styleSheet = this.getStyleSheet(styleSheet);
        CSS.LengthValue lengthValue = (CSS.LengthValue)set.getAttribute(attribute);
        int n = styleSheet != null && styleSheet.isW3CLengthUnits();
        return (lengthValue != null) ? lengthValue.getValue((boolean)n) : 0.0f;
    }
    
    AttributeSet translateHTMLToCSS(AttributeSet set) {
        SimpleAttributeSet set2 = new SimpleAttributeSet();
        Element element = (Element)set;
        HTML.Tag htmlTag = this.getHTMLTag(set);
        if (htmlTag == HTML.Tag.TD || htmlTag == HTML.Tag.TH) {
            AttributeSet attributes = element.getParentElement().getParentElement().getAttributes();
            if (getTableBorder(attributes) > 0) {
                this.translateAttribute(HTML.Attribute.BORDER, "1", set2);
            }
            String s = (String)attributes.getAttribute(HTML.Attribute.CELLPADDING);
            if (s != null) {
                CSS.LengthValue lengthValue = (CSS.LengthValue)this.getInternalCSSValue(CSS.Attribute.PADDING_TOP, s);
                lengthValue.span = ((lengthValue.span < 0.0f) ? 0.0f : lengthValue.span);
                set2.addAttribute(CSS.Attribute.PADDING_TOP, lengthValue);
                set2.addAttribute(CSS.Attribute.PADDING_BOTTOM, lengthValue);
                set2.addAttribute(CSS.Attribute.PADDING_LEFT, lengthValue);
                set2.addAttribute(CSS.Attribute.PADDING_RIGHT, lengthValue);
            }
        }
        if (element.isLeaf()) {
            this.translateEmbeddedAttributes(set, set2);
        }
        else {
            this.translateAttributes(htmlTag, set, set2);
        }
        if (htmlTag == HTML.Tag.CAPTION) {
            Object attribute = set.getAttribute(HTML.Attribute.ALIGN);
            if (attribute != null && (attribute.equals("top") || attribute.equals("bottom"))) {
                set2.addAttribute(CSS.Attribute.CAPTION_SIDE, attribute);
                set2.removeAttribute(CSS.Attribute.TEXT_ALIGN);
            }
            else {
                Object attribute2 = set.getAttribute(HTML.Attribute.VALIGN);
                if (attribute2 != null) {
                    set2.addAttribute(CSS.Attribute.CAPTION_SIDE, attribute2);
                }
            }
        }
        return set2;
    }
    
    private static int getTableBorder(AttributeSet set) {
        String s = (String)set.getAttribute(HTML.Attribute.BORDER);
        if (s == "#DEFAULT" || "".equals(s)) {
            return 1;
        }
        try {
            return Integer.parseInt(s);
        }
        catch (NumberFormatException ex) {
            return 0;
        }
    }
    
    public static CSS.Attribute[] getAllAttributeKeys() {
        CSS.Attribute[] array = new CSS.Attribute[CSS.Attribute.allAttributes.length];
        System.arraycopy(CSS.Attribute.allAttributes, 0, array, 0, CSS.Attribute.allAttributes.length);
        return array;
    }
    
    public static final CSS.Attribute getAttribute(String key) {
        return (CSS.Attribute)CSS.attributeMap.get(key);
    }
    
    static final CSS.Value getValue(String key) {
        return (CSS.Value)CSS.valueMap.get(key);
    }
    
    static URL getURL(URL context, String substring) {
        if (substring == null) {
            return null;
        }
        if (substring.startsWith("url(") && substring.endsWith(")")) {
            substring = substring.substring(4, substring.length() - 1);
        }
        try {
            URL o = new URL(substring);
            if (o != null) {
                return o;
            }
        }
        catch (MalformedURLException ex) {}
        if (context != null) {
            try {
                return new URL(context, substring);
            }
            catch (MalformedURLException ex2) {}
        }
        return null;
    }
    
    static String colorToHex(Color color) {
        String s = "#";
        String hexString = Integer.toHexString(color.getRed());
        if (hexString.length() > 2) {
            hexString.substring(0, 2);
        }
        else if (hexString.length() < 2) {
            s = s + "0" + hexString;
        }
        else {
            s += hexString;
        }
        String hexString2 = Integer.toHexString(color.getGreen());
        if (hexString2.length() > 2) {
            hexString2.substring(0, 2);
        }
        else if (hexString2.length() < 2) {
            s = s + "0" + hexString2;
        }
        else {
            s += hexString2;
        }
        String hexString3 = Integer.toHexString(color.getBlue());
        if (hexString3.length() > 2) {
            hexString3.substring(0, 2);
        }
        else if (hexString3.length() < 2) {
            s = s + "0" + hexString3;
        }
        else {
            s += hexString3;
        }
        return s;
    }
    
    static final Color hexToColor(String s) {
        s.length();
        String substring;
        if (s.startsWith("#")) {
            substring = s.substring(1, Math.min(s.length(), 7));
        }
        else {
            substring = s;
        }
        String string = "0x" + substring;
        Color decode;
        try {
            decode = Color.decode(string);
        }
        catch (NumberFormatException ex) {
            decode = null;
        }
        return decode;
    }
    
    static Color stringToColor(String s) {
        if (s == null) {
            return null;
        }
        Color color;
        if (s.length() == 0) {
            color = Color.black;
        }
        else if (s.startsWith("rgb(")) {
            color = parseRGB(s);
        }
        else if (s.charAt(0) == '#') {
            color = hexToColor(s);
        }
        else if (s.equalsIgnoreCase("Black")) {
            color = hexToColor("#000000");
        }
        else if (s.equalsIgnoreCase("Silver")) {
            color = hexToColor("#C0C0C0");
        }
        else if (s.equalsIgnoreCase("Gray")) {
            color = hexToColor("#808080");
        }
        else if (s.equalsIgnoreCase("White")) {
            color = hexToColor("#FFFFFF");
        }
        else if (s.equalsIgnoreCase("Maroon")) {
            color = hexToColor("#800000");
        }
        else if (s.equalsIgnoreCase("Red")) {
            color = hexToColor("#FF0000");
        }
        else if (s.equalsIgnoreCase("Purple")) {
            color = hexToColor("#800080");
        }
        else if (s.equalsIgnoreCase("Fuchsia")) {
            color = hexToColor("#FF00FF");
        }
        else if (s.equalsIgnoreCase("Green")) {
            color = hexToColor("#008000");
        }
        else if (s.equalsIgnoreCase("Lime")) {
            color = hexToColor("#00FF00");
        }
        else if (s.equalsIgnoreCase("Olive")) {
            color = hexToColor("#808000");
        }
        else if (s.equalsIgnoreCase("Yellow")) {
            color = hexToColor("#FFFF00");
        }
        else if (s.equalsIgnoreCase("Navy")) {
            color = hexToColor("#000080");
        }
        else if (s.equalsIgnoreCase("Blue")) {
            color = hexToColor("#0000FF");
        }
        else if (s.equalsIgnoreCase("Teal")) {
            color = hexToColor("#008080");
        }
        else if (s.equalsIgnoreCase("Aqua")) {
            color = hexToColor("#00FFFF");
        }
        else if (s.equalsIgnoreCase("Orange")) {
            color = hexToColor("#FF8000");
        }
        else {
            color = hexToColor(s);
        }
        return color;
    }
    
    private static Color parseRGB(String s) {
        int[] array = { 4 };
        return new Color(getColorComponent(s, array), getColorComponent(s, array), getColorComponent(s, array));
    }
    
    private static int getColorComponent(String s, int[] array) {
        int length = s.length();
        int char1;
        while (array[0] < length && (char1 = s.charAt(array[0])) != '-' && !Character.isDigit((char)char1) && char1 != '.') {
            int n = 0;
            ++array[n];
        }
        int beginIndex = array[0];
        if (beginIndex < length && s.charAt(array[0]) == '-') {
            int n2 = 0;
            ++array[n2];
        }
        while (array[0] < length && Character.isDigit(s.charAt(array[0]))) {
            int n3 = 0;
            ++array[n3];
        }
        if (array[0] < length && s.charAt(array[0]) == '.') {
            int n4 = 0;
            ++array[n4];
            while (array[0] < length && Character.isDigit(s.charAt(array[0]))) {
                int n5 = 0;
                ++array[n5];
            }
        }
        if (beginIndex != array[0]) {
            try {
                float float1 = Float.parseFloat(s.substring(beginIndex, array[0]));
                if (array[0] < length && s.charAt(array[0]) == '%') {
                    int n6 = 0;
                    ++array[n6];
                    float1 = float1 * 255.0f / 100.0f;
                }
                return Math.min(255, Math.max(0, (int)float1));
            }
            catch (NumberFormatException ex) {}
        }
        return 0;
    }
    
    static int getIndexOfSize(float n, int[] array) {
        int i = 0;
        while (i < array.length) {
            if (n <= (float)array[i]) {
                return i + 1;
            }
            ++i;
        }
        return array.length;
    }
    
    static int getIndexOfSize(float n, StyleSheet styleSheet) {
        return getIndexOfSize(n, (styleSheet != null) ? styleSheet.getSizeMap() : StyleSheet.sizeMapDefault);
    }
    
    static String[] parseStrings(String s) {
        int n = (s == null) ? 0 : s.length();
        Vector vector = new Vector(4);
        for (int i = 0; i < n; ++i) {
            while (i < n && Character.isWhitespace(s.charAt(i))) {
                ++i;
            }
            for (; i < n && !Character.isWhitespace(s.charAt(i)); ++i) {}
            if (i != i) {
                vector.addElement(s.substring(i, i));
            }
        }
        String[] anArray = new String[vector.size()];
        vector.copyInto(anArray);
        return anArray;
    }
    
    float getPointSize(int n, StyleSheet styleSheet) {
        styleSheet = this.getStyleSheet(styleSheet);
        int[] array = (styleSheet != null) ? styleSheet.getSizeMap() : StyleSheet.sizeMapDefault;
        if (--n < 0) {
            return (float)array[0];
        }
        if (n > array.length - 1) {
            return (float)array[array.length - 1];
        }
        return (float)array[n];
    }
    
    private void translateEmbeddedAttributes(AttributeSet set, MutableAttributeSet set2) {
        Enumeration<Object> attributeNames = set.getAttributeNames();
        if (set.getAttribute(StyleConstants.NameAttribute) == HTML.Tag.HR) {
            this.translateAttributes(HTML.Tag.HR, set, set2);
        }
        while (attributeNames.hasMoreElements()) {
            Object nextElement = attributeNames.nextElement();
            if ((nextElement instanceof HTML.Tag)) {
                HTML.Tag tag = (HTML.Tag)nextElement;
                Object attribute = set.getAttribute(tag);
                if (attribute != null && attribute instanceof AttributeSet) {
                    this.translateAttributes(tag, (AttributeSet)attribute, set2);
                }
                else {
                    continue;
                }
            }
            else if ((nextElement instanceof CSS.Attribute)) {
                set2.addAttribute(nextElement, set.getAttribute(nextElement));
            }
            else {
                continue;
            }
        }
    }
    
    private void translateAttributes(HTML.Tag tag, AttributeSet set, MutableAttributeSet set2) {
        Enumeration<Object> attributeNames = set.getAttributeNames();
        while (attributeNames.hasMoreElements()) {
            Object nextElement = attributeNames.nextElement();
            if ((nextElement instanceof HTML.Attribute)) {
                HTML.Attribute attribute = (HTML.Attribute)nextElement;
                if (attribute == HTML.Attribute.ALIGN) {
                    String s = (String)set.getAttribute(HTML.Attribute.ALIGN);
                    if (s != null) {
                        CSS.Attribute cssAlignAttribute = this.getCssAlignAttribute(tag, set);
                        if (cssAlignAttribute != null) {
                            Object cssValue = this.getCssValue(cssAlignAttribute, s);
                            if (cssValue != null) {
                                set2.addAttribute(cssAlignAttribute, cssValue);
                            }
                            else {
                                continue;
                            }
                        }
                        else {
                            continue;
                        }
                    }
                    else {
                        continue;
                    }
                }
                else {
                    if (attribute == HTML.Attribute.SIZE && !this.isHTMLFontTag(tag)) {
                        continue;
                    }
                    if (tag == HTML.Tag.TABLE && attribute == HTML.Attribute.BORDER) {
                        int tableBorder = getTableBorder(set);
                        if (tableBorder > 0) {
                            this.translateAttribute(HTML.Attribute.BORDER, Integer.toString(tableBorder), set2);
                        }
                        else {
                            continue;
                        }
                    }
                    else {
                        this.translateAttribute(attribute, (String)set.getAttribute(attribute), set2);
                    }
                }
            }
            else if ((nextElement instanceof CSS.Attribute)) {
                set2.addAttribute(nextElement, set.getAttribute(nextElement));
            }
            else {
                continue;
            }
        }
    }
    
    private void translateAttribute(HTML.Attribute attribute, String s, MutableAttributeSet set) {
        CSS.Attribute[] cssAttribute = this.getCssAttribute(attribute);
        if (cssAttribute == null || s == null) {
            return;
        }
        for (CSS.Attribute attribute2 : cssAttribute) {
            Object cssValue = this.getCssValue(attribute2, s);
            if (cssValue != null) {
                set.addAttribute(attribute2, cssValue);
            }
        }
        return;
    }
    
    Object getCssValue(CSS.Attribute key, String s) {
        return ((CSS.CssValue)this.valueConvertor.get(key)).parseHtmlValue(s);
    }
    
    private CSS.Attribute[] getCssAttribute(HTML.Attribute key) {
        return (CSS.Attribute[])CSS.htmlAttrToCssAttrMap.get(key);
    }
    
    private CSS.Attribute getCssAlignAttribute(HTML.Tag tag, AttributeSet set) {
        return CSS.Attribute.TEXT_ALIGN;
    }
    
    private HTML.Tag getHTMLTag(AttributeSet set) {
        Object attribute = set.getAttribute(StyleConstants.NameAttribute);
        if ((attribute instanceof HTML.Tag)) {
            return (HTML.Tag)attribute;
        }
        return null;
    }
    
    private boolean isHTMLFontTag(HTML.Tag tag) {
        return tag != null && (tag == HTML.Tag.FONT || tag == HTML.Tag.BASEFONT);
    }
    
    private boolean isFloater(String s) {
        return s.equals("left") || s.equals("right");
    }
    
    private boolean validTextAlignValue(String s) {
        return this.isFloater(s) || s.equals("center");
    }
    
    static SizeRequirements calculateTiledRequirements(CSS.LayoutIterator layoutIterator, SizeRequirements sizeRequirements) {
        long n = 0L;
        long n2 = 0L;
        long n3 = 0L;
        int a = 0;
        int n4 = 0;
        for (int count = layoutIterator.getCount(), i = 0; i < count; , n4 += Math.max(a, (int)layoutIterator.getLeadingCollapseSpan()), n3 += (long)(int)layoutIterator.getPreferredSpan(0.0f), n = (long)((float)n + layoutIterator.getMinimumSpan(0.0f)), n2 = (long)((float)n2 + layoutIterator.getMaximumSpan(0.0f)), a = (int)layoutIterator.getTrailingCollapseSpan(), ++i) {
            layoutIterator.setIndex(i);
        }
        int n5 = (int)((float)(n4 + a) + 2.0f * layoutIterator.getBorderWidth());
        long n6 = n + (long)n5;
        long n7 = n3 + (long)n5;
        long n8 = n2 + (long)n5;
        if (sizeRequirements == null) {
            sizeRequirements = new SizeRequirements();
        }
        sizeRequirements.minimum = ((n6 > 2147483647L) ? Integer.MAX_VALUE : ((int)n6));
        sizeRequirements.preferred = ((n7 > 2147483647L) ? Integer.MAX_VALUE : ((int)n7));
        sizeRequirements.maximum = ((n8 > 2147483647L) ? Integer.MAX_VALUE : ((int)n8));
        return sizeRequirements;
    }
    
    static void calculateTiledLayout(CSS.LayoutIterator layoutIterator, int n) {
        long n2 = 0L;
        int a = 0;
        int n3 = 0;
        int count = layoutIterator.getCount();
        int n4 = 3;
        long[] array = new long[n4];
        long[] array2 = new long[n4];
        for (int i = 0; i < n4; ++i) {
            array[i] = (array2[i] = 0L);
        }
        long n5;
        int adjustmentWeight;
        int adjustmentWeight2;
        for (int j = 0; j < count; n2 += n5, adjustmentWeight = layoutIterator.getAdjustmentWeight(), array[adjustmentWeight] += (long)layoutIterator.getMaximumSpan((float)n) - n5, adjustmentWeight2 = layoutIterator.getAdjustmentWeight(), array2[adjustmentWeight2] += n5 - (long)layoutIterator.getMinimumSpan((float)n), a = (int)layoutIterator.getTrailingCollapseSpan(), ++j) {
            layoutIterator.setIndex(j);
            layoutIterator.setOffset(Math.max(a, (int)layoutIterator.getLeadingCollapseSpan()));
            n3 += layoutIterator.getOffset();
            n5 = (long)layoutIterator.getPreferredSpan((float)n);
            layoutIterator.setSpan((int)n5);
        }
        int n6 = (int)((float)(n3 + a) + 2.0f * layoutIterator.getBorderWidth());
        for (int k = 1; k < n4; ++k) {
            int n7 = k;
            array[n7] += array[k - 1];
            int n8 = k;
            array2[n8] += array2[k - 1];
        }
        int n9 = n - n6;
        long a2 = (long)n9 - n2;
        long abs;
        int n10;
        for (array2 = ((a2 > 0L) ? array : array2), abs = Math.abs(a2), n10 = 0; n10 <= 2 && array2[n10] < abs; ++n10) {}
        float n11 = 0.0f;
        if (n10 <= 2) {
            long n12 = abs - ((n10 > 0) ? array2[n10 - 1] : 0L);
            if (n12 != 0L) {
                n11 = (float)n12 / (float)(array2[n10] - ((n10 > 0) ? array2[n10 - 1] : 0L));
            }
        }
        int n13 = (int)layoutIterator.getBorderWidth();
        for (int l = 0; l < count; n13 = (int)Math.min((long)layoutIterator.getOffset() + (long)layoutIterator.getSpan(), 2147483647L), ++l) {
            layoutIterator.setIndex(l);
            layoutIterator.setOffset(layoutIterator.getOffset() + n13);
            if (layoutIterator.getAdjustmentWeight() < n10) {
                layoutIterator.setSpan((int)(((long)n9 > n2) ? Math.floor((double)layoutIterator.getMaximumSpan((float)n)) : Math.ceil((double)layoutIterator.getMinimumSpan((float)n))));
            }
            else if (layoutIterator.getAdjustmentWeight() == n10) {
                int n14 = (int)Math.floor((double)(n11 * (float)(((long)n9 > n2) ? ((int)layoutIterator.getMaximumSpan((float)n) - layoutIterator.getSpan()) : (layoutIterator.getSpan() - (int)layoutIterator.getMinimumSpan((float)n)))));
                layoutIterator.setSpan(layoutIterator.getSpan() + (((long)n9 > n2) ? n14 : (-n14)));
            }
        }
        int n15 = n - n13 - (int)layoutIterator.getTrailingCollapseSpan() - (int)layoutIterator.getBorderWidth();
        int n16 = (n15 > 0) ? 1 : -1;
        int n17 = n15 * n16;
        int n18 = 1;
        while (n17 > 0 && n18 != 0) {
            n18 = 0;
            int n19 = 0;
            for (int index = 0; index < count; ++index) {
                layoutIterator.setIndex(index);
                layoutIterator.setOffset(layoutIterator.getOffset() + n19);
                int span = layoutIterator.getSpan();
                if (n17 > 0 && ((n16 > 0) ? ((int)Math.floor((double)layoutIterator.getMaximumSpan((float)n)) - span) : (span - (int)Math.ceil((double)layoutIterator.getMinimumSpan((float)n)))) >= 1) {
                    n18 = 1;
                    layoutIterator.setSpan(span + n16);
                    n19 += n16;
                    --n17;
                }
            }
        }
    }
    
    private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        objectOutputStream.defaultWriteObject();
        Enumeration<Object> keys = this.valueConvertor.keys();
        objectOutputStream.writeInt(this.valueConvertor.size());
        if (keys != null) {
            while (keys.hasMoreElements()) {
                Object obj = keys.nextElement();
                Object o = this.valueConvertor.get(obj);
                if (!(obj instanceof Serializable) && (obj = StyleContext.getStaticAttributeKey(obj)) == null) {
                    obj = null;
                    o = null;
                }
                else if (!(o instanceof Serializable) && (o = StyleContext.getStaticAttributeKey(o)) == null) {
                    obj = null;
                    o = null;
                }
                objectOutputStream.writeObject(obj);
                objectOutputStream.writeObject(o);
            }
        }
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws ClassNotFoundException, IOException {
        objectInputStream.defaultReadObject();
        int int1 = objectInputStream.readInt();
        this.valueConvertor = new Hashtable();
        while (int1-- > 0) {
            Object object = objectInputStream.readObject();
            Object object2 = objectInputStream.readObject();
            Object staticAttribute = StyleContext.getStaticAttribute(object);
            if (staticAttribute != null) {
                object = staticAttribute;
            }
            Object staticAttribute2 = StyleContext.getStaticAttribute(object2);
            if (staticAttribute2 != null) {
                object2 = staticAttribute2;
            }
            if (object != null && object2 != null) {
                this.valueConvertor.put(object, object2);
            }
            else {
                continue;
            }
        }
    }
    
    private StyleSheet getStyleSheet(StyleSheet styleSheet) {
        if (styleSheet != null) {
            this.styleSheet = styleSheet;
        }
        return this.styleSheet;
    }
    
    static StyleSheet access$500(CSS css, StyleSheet styleSheet) {
        return css.getStyleSheet(styleSheet);
    }
    
    static {
        attributeMap = new Hashtable();
        valueMap = new Hashtable();
        htmlAttrToCssAttrMap = new Hashtable(20);
        styleConstantToCssMap = new Hashtable(17);
        htmlValueToCssValueMap = new Hashtable(8);
        cssValueToInternalValueMap = new Hashtable(13);
        for (int i = 0; i < CSS.Attribute.allAttributes.length; ++i) {
            CSS.attributeMap.put(CSS.Attribute.allAttributes[i].toString(), CSS.Attribute.allAttributes[i]);
        }
        for (int j = 0; j < CSS.Value.allValues.length; ++j) {
            CSS.valueMap.put(CSS.Value.allValues[j].toString(), CSS.Value.allValues[j]);
        }
        CSS.htmlAttrToCssAttrMap.put(HTML.Attribute.COLOR, new CSS.Attribute[] { CSS.Attribute.COLOR });
        CSS.htmlAttrToCssAttrMap.put(HTML.Attribute.TEXT, new CSS.Attribute[] { CSS.Attribute.COLOR });
        CSS.htmlAttrToCssAttrMap.put(HTML.Attribute.CLEAR, new CSS.Attribute[] { CSS.Attribute.CLEAR });
        CSS.htmlAttrToCssAttrMap.put(HTML.Attribute.BACKGROUND, new CSS.Attribute[] { CSS.Attribute.BACKGROUND_IMAGE });
        CSS.htmlAttrToCssAttrMap.put(HTML.Attribute.BGCOLOR, new CSS.Attribute[] { CSS.Attribute.BACKGROUND_COLOR });
        CSS.htmlAttrToCssAttrMap.put(HTML.Attribute.WIDTH, new CSS.Attribute[] { CSS.Attribute.WIDTH });
        CSS.htmlAttrToCssAttrMap.put(HTML.Attribute.HEIGHT, new CSS.Attribute[] { CSS.Attribute.HEIGHT });
        CSS.htmlAttrToCssAttrMap.put(HTML.Attribute.BORDER, new CSS.Attribute[] { CSS.Attribute.BORDER_TOP_WIDTH, CSS.Attribute.BORDER_RIGHT_WIDTH, CSS.Attribute.BORDER_BOTTOM_WIDTH, CSS.Attribute.BORDER_LEFT_WIDTH });
        CSS.htmlAttrToCssAttrMap.put(HTML.Attribute.CELLPADDING, new CSS.Attribute[] { CSS.Attribute.PADDING });
        CSS.htmlAttrToCssAttrMap.put(HTML.Attribute.CELLSPACING, new CSS.Attribute[] { CSS.Attribute.BORDER_SPACING });
        CSS.htmlAttrToCssAttrMap.put(HTML.Attribute.MARGINWIDTH, new CSS.Attribute[] { CSS.Attribute.MARGIN_LEFT, CSS.Attribute.MARGIN_RIGHT });
        CSS.htmlAttrToCssAttrMap.put(HTML.Attribute.MARGINHEIGHT, new CSS.Attribute[] { CSS.Attribute.MARGIN_TOP, CSS.Attribute.MARGIN_BOTTOM });
        CSS.htmlAttrToCssAttrMap.put(HTML.Attribute.HSPACE, new CSS.Attribute[] { CSS.Attribute.PADDING_LEFT, CSS.Attribute.PADDING_RIGHT });
        CSS.htmlAttrToCssAttrMap.put(HTML.Attribute.VSPACE, new CSS.Attribute[] { CSS.Attribute.PADDING_BOTTOM, CSS.Attribute.PADDING_TOP });
        CSS.htmlAttrToCssAttrMap.put(HTML.Attribute.FACE, new CSS.Attribute[] { CSS.Attribute.FONT_FAMILY });
        CSS.htmlAttrToCssAttrMap.put(HTML.Attribute.SIZE, new CSS.Attribute[] { CSS.Attribute.FONT_SIZE });
        CSS.htmlAttrToCssAttrMap.put(HTML.Attribute.VALIGN, new CSS.Attribute[] { CSS.Attribute.VERTICAL_ALIGN });
        CSS.htmlAttrToCssAttrMap.put(HTML.Attribute.ALIGN, new CSS.Attribute[] { CSS.Attribute.VERTICAL_ALIGN, CSS.Attribute.TEXT_ALIGN, CSS.Attribute.FLOAT });
        CSS.htmlAttrToCssAttrMap.put(HTML.Attribute.TYPE, new CSS.Attribute[] { CSS.Attribute.LIST_STYLE_TYPE });
        CSS.htmlAttrToCssAttrMap.put(HTML.Attribute.NOWRAP, new CSS.Attribute[] { CSS.Attribute.WHITE_SPACE });
        CSS.styleConstantToCssMap.put(StyleConstants.FontFamily, CSS.Attribute.FONT_FAMILY);
        CSS.styleConstantToCssMap.put(StyleConstants.FontSize, CSS.Attribute.FONT_SIZE);
        CSS.styleConstantToCssMap.put(StyleConstants.Bold, CSS.Attribute.FONT_WEIGHT);
        CSS.styleConstantToCssMap.put(StyleConstants.Italic, CSS.Attribute.FONT_STYLE);
        CSS.styleConstantToCssMap.put(StyleConstants.Underline, CSS.Attribute.TEXT_DECORATION);
        CSS.styleConstantToCssMap.put(StyleConstants.StrikeThrough, CSS.Attribute.TEXT_DECORATION);
        CSS.styleConstantToCssMap.put(StyleConstants.Superscript, CSS.Attribute.VERTICAL_ALIGN);
        CSS.styleConstantToCssMap.put(StyleConstants.Subscript, CSS.Attribute.VERTICAL_ALIGN);
        CSS.styleConstantToCssMap.put(StyleConstants.Foreground, CSS.Attribute.COLOR);
        CSS.styleConstantToCssMap.put(StyleConstants.Background, CSS.Attribute.BACKGROUND_COLOR);
        CSS.styleConstantToCssMap.put(StyleConstants.FirstLineIndent, CSS.Attribute.TEXT_INDENT);
        CSS.styleConstantToCssMap.put(StyleConstants.LeftIndent, CSS.Attribute.MARGIN_LEFT);
        CSS.styleConstantToCssMap.put(StyleConstants.RightIndent, CSS.Attribute.MARGIN_RIGHT);
        CSS.styleConstantToCssMap.put(StyleConstants.SpaceAbove, CSS.Attribute.MARGIN_TOP);
        CSS.styleConstantToCssMap.put(StyleConstants.SpaceBelow, CSS.Attribute.MARGIN_BOTTOM);
        CSS.styleConstantToCssMap.put(StyleConstants.Alignment, CSS.Attribute.TEXT_ALIGN);
        CSS.htmlValueToCssValueMap.put("disc", CSS.Value.DISC);
        CSS.htmlValueToCssValueMap.put("square", CSS.Value.SQUARE);
        CSS.htmlValueToCssValueMap.put("circle", CSS.Value.CIRCLE);
        CSS.htmlValueToCssValueMap.put("1", CSS.Value.DECIMAL);
        CSS.htmlValueToCssValueMap.put("a", CSS.Value.LOWER_ALPHA);
        CSS.htmlValueToCssValueMap.put("A", CSS.Value.UPPER_ALPHA);
        CSS.htmlValueToCssValueMap.put("i", CSS.Value.LOWER_ROMAN);
        CSS.htmlValueToCssValueMap.put("I", CSS.Value.UPPER_ROMAN);
        CSS.cssValueToInternalValueMap.put("none", CSS.Value.NONE);
        CSS.cssValueToInternalValueMap.put("disc", CSS.Value.DISC);
        CSS.cssValueToInternalValueMap.put("square", CSS.Value.SQUARE);
        CSS.cssValueToInternalValueMap.put("circle", CSS.Value.CIRCLE);
        CSS.cssValueToInternalValueMap.put("decimal", CSS.Value.DECIMAL);
        CSS.cssValueToInternalValueMap.put("lower-roman", CSS.Value.LOWER_ROMAN);
        CSS.cssValueToInternalValueMap.put("upper-roman", CSS.Value.UPPER_ROMAN);
        CSS.cssValueToInternalValueMap.put("lower-alpha", CSS.Value.LOWER_ALPHA);
        CSS.cssValueToInternalValueMap.put("upper-alpha", CSS.Value.UPPER_ALPHA);
        CSS.cssValueToInternalValueMap.put("repeat", CSS.Value.BACKGROUND_REPEAT);
        CSS.cssValueToInternalValueMap.put("no-repeat", CSS.Value.BACKGROUND_NO_REPEAT);
        CSS.cssValueToInternalValueMap.put("repeat-x", CSS.Value.BACKGROUND_REPEAT_X);
        CSS.cssValueToInternalValueMap.put("repeat-y", CSS.Value.BACKGROUND_REPEAT_Y);
        CSS.cssValueToInternalValueMap.put("scroll", CSS.Value.BACKGROUND_SCROLL);
        CSS.cssValueToInternalValueMap.put("fixed", CSS.Value.BACKGROUND_FIXED);
        CSS.Attribute[] allAttributes = CSS.Attribute.allAttributes;
        try {
            for (int length = allAttributes.length, k = 0; k < length; ++k) {
                StyleContext.registerStaticAttributeKey(allAttributes[k]);
            }
        }
        catch (Throwable t) {
            t.printStackTrace();
        }
        CSS.Value[] allValues = CSS.Value.allValues;
        try {
            for (int length2 = allValues.length, l = 0; l < length2; ++l) {
                StyleContext.registerStaticAttributeKey(allValues[l]);
            }
        }
        catch (Throwable t2) {
            t2.printStackTrace();
        }
        CSS.baseFontSizeIndex = 3;
    }
    
    public static final class Attribute
    {
        private String name;
        private String defaultValue;
        private boolean inherited;
        public static final CSS.Attribute BACKGROUND;
        public static final CSS.Attribute BACKGROUND_ATTACHMENT;
        public static final CSS.Attribute BACKGROUND_COLOR;
        public static final CSS.Attribute BACKGROUND_IMAGE;
        public static final CSS.Attribute BACKGROUND_POSITION;
        public static final CSS.Attribute BACKGROUND_REPEAT;
        public static final CSS.Attribute BORDER;
        public static final CSS.Attribute BORDER_BOTTOM;
        public static final CSS.Attribute BORDER_BOTTOM_COLOR;
        public static final CSS.Attribute BORDER_BOTTOM_STYLE;
        public static final CSS.Attribute BORDER_BOTTOM_WIDTH;
        public static final CSS.Attribute BORDER_COLOR;
        public static final CSS.Attribute BORDER_LEFT;
        public static final CSS.Attribute BORDER_LEFT_COLOR;
        public static final CSS.Attribute BORDER_LEFT_STYLE;
        public static final CSS.Attribute BORDER_LEFT_WIDTH;
        public static final CSS.Attribute BORDER_RIGHT;
        public static final CSS.Attribute BORDER_RIGHT_COLOR;
        public static final CSS.Attribute BORDER_RIGHT_STYLE;
        public static final CSS.Attribute BORDER_RIGHT_WIDTH;
        public static final CSS.Attribute BORDER_STYLE;
        public static final CSS.Attribute BORDER_TOP;
        public static final CSS.Attribute BORDER_TOP_COLOR;
        public static final CSS.Attribute BORDER_TOP_STYLE;
        public static final CSS.Attribute BORDER_TOP_WIDTH;
        public static final CSS.Attribute BORDER_WIDTH;
        public static final CSS.Attribute CLEAR;
        public static final CSS.Attribute COLOR;
        public static final CSS.Attribute DISPLAY;
        public static final CSS.Attribute FLOAT;
        public static final CSS.Attribute FONT;
        public static final CSS.Attribute FONT_FAMILY;
        public static final CSS.Attribute FONT_SIZE;
        public static final CSS.Attribute FONT_STYLE;
        public static final CSS.Attribute FONT_VARIANT;
        public static final CSS.Attribute FONT_WEIGHT;
        public static final CSS.Attribute HEIGHT;
        public static final CSS.Attribute LETTER_SPACING;
        public static final CSS.Attribute LINE_HEIGHT;
        public static final CSS.Attribute LIST_STYLE;
        public static final CSS.Attribute LIST_STYLE_IMAGE;
        public static final CSS.Attribute LIST_STYLE_POSITION;
        public static final CSS.Attribute LIST_STYLE_TYPE;
        public static final CSS.Attribute MARGIN;
        public static final CSS.Attribute MARGIN_BOTTOM;
        public static final CSS.Attribute MARGIN_LEFT;
        public static final CSS.Attribute MARGIN_RIGHT;
        static final CSS.Attribute MARGIN_LEFT_LTR;
        static final CSS.Attribute MARGIN_LEFT_RTL;
        static final CSS.Attribute MARGIN_RIGHT_LTR;
        static final CSS.Attribute MARGIN_RIGHT_RTL;
        public static final CSS.Attribute MARGIN_TOP;
        public static final CSS.Attribute PADDING;
        public static final CSS.Attribute PADDING_BOTTOM;
        public static final CSS.Attribute PADDING_LEFT;
        public static final CSS.Attribute PADDING_RIGHT;
        public static final CSS.Attribute PADDING_TOP;
        public static final CSS.Attribute TEXT_ALIGN;
        public static final CSS.Attribute TEXT_DECORATION;
        public static final CSS.Attribute TEXT_INDENT;
        public static final CSS.Attribute TEXT_TRANSFORM;
        public static final CSS.Attribute VERTICAL_ALIGN;
        public static final CSS.Attribute WORD_SPACING;
        public static final CSS.Attribute WHITE_SPACE;
        public static final CSS.Attribute WIDTH;
        static final CSS.Attribute BORDER_SPACING;
        static final CSS.Attribute CAPTION_SIDE;
        static final CSS.Attribute[] allAttributes;
        private static final CSS.Attribute[] ALL_MARGINS;
        private static final CSS.Attribute[] ALL_PADDING;
        private static final CSS.Attribute[] ALL_BORDER_WIDTHS;
        private static final CSS.Attribute[] ALL_BORDER_STYLES;
        private static final CSS.Attribute[] ALL_BORDER_COLORS;
        
        private Attribute(String name, String defaultValue, boolean inherited) {
            this.name = name;
            this.defaultValue = defaultValue;
            this.inherited = inherited;
        }
        
        @Override
        public String toString() {
            return this.name;
        }
        
        public String getDefaultValue() {
            return this.defaultValue;
        }
        
        public boolean isInherited() {
            return this.inherited;
        }
        
        static {
            BACKGROUND = new CSS.Attribute("background", null, false);
            BACKGROUND_ATTACHMENT = new CSS.Attribute("background-attachment", "scroll", false);
            BACKGROUND_COLOR = new CSS.Attribute("background-color", "transparent", false);
            BACKGROUND_IMAGE = new CSS.Attribute("background-image", "none", false);
            BACKGROUND_POSITION = new CSS.Attribute("background-position", null, false);
            BACKGROUND_REPEAT = new CSS.Attribute("background-repeat", "repeat", false);
            BORDER = new CSS.Attribute("border", null, false);
            BORDER_BOTTOM = new CSS.Attribute("border-bottom", null, false);
            BORDER_BOTTOM_COLOR = new CSS.Attribute("border-bottom-color", null, false);
            BORDER_BOTTOM_STYLE = new CSS.Attribute("border-bottom-style", "none", false);
            BORDER_BOTTOM_WIDTH = new CSS.Attribute("border-bottom-width", "medium", false);
            BORDER_COLOR = new CSS.Attribute("border-color", null, false);
            BORDER_LEFT = new CSS.Attribute("border-left", null, false);
            BORDER_LEFT_COLOR = new CSS.Attribute("border-left-color", null, false);
            BORDER_LEFT_STYLE = new CSS.Attribute("border-left-style", "none", false);
            BORDER_LEFT_WIDTH = new CSS.Attribute("border-left-width", "medium", false);
            BORDER_RIGHT = new CSS.Attribute("border-right", null, false);
            BORDER_RIGHT_COLOR = new CSS.Attribute("border-right-color", null, false);
            BORDER_RIGHT_STYLE = new CSS.Attribute("border-right-style", "none", false);
            BORDER_RIGHT_WIDTH = new CSS.Attribute("border-right-width", "medium", false);
            BORDER_STYLE = new CSS.Attribute("border-style", "none", false);
            BORDER_TOP = new CSS.Attribute("border-top", null, false);
            BORDER_TOP_COLOR = new CSS.Attribute("border-top-color", null, false);
            BORDER_TOP_STYLE = new CSS.Attribute("border-top-style", "none", false);
            BORDER_TOP_WIDTH = new CSS.Attribute("border-top-width", "medium", false);
            BORDER_WIDTH = new CSS.Attribute("border-width", "medium", false);
            CLEAR = new CSS.Attribute("clear", "none", false);
            COLOR = new CSS.Attribute("color", "black", true);
            DISPLAY = new CSS.Attribute("display", "block", false);
            FLOAT = new CSS.Attribute("float", "none", false);
            FONT = new CSS.Attribute("font", null, true);
            FONT_FAMILY = new CSS.Attribute("font-family", null, true);
            FONT_SIZE = new CSS.Attribute("font-size", "medium", true);
            FONT_STYLE = new CSS.Attribute("font-style", "normal", true);
            FONT_VARIANT = new CSS.Attribute("font-variant", "normal", true);
            FONT_WEIGHT = new CSS.Attribute("font-weight", "normal", true);
            HEIGHT = new CSS.Attribute("height", "auto", false);
            LETTER_SPACING = new CSS.Attribute("letter-spacing", "normal", true);
            LINE_HEIGHT = new CSS.Attribute("line-height", "normal", true);
            LIST_STYLE = new CSS.Attribute("list-style", null, true);
            LIST_STYLE_IMAGE = new CSS.Attribute("list-style-image", "none", true);
            LIST_STYLE_POSITION = new CSS.Attribute("list-style-position", "outside", true);
            LIST_STYLE_TYPE = new CSS.Attribute("list-style-type", "disc", true);
            MARGIN = new CSS.Attribute("margin", null, false);
            MARGIN_BOTTOM = new CSS.Attribute("margin-bottom", "0", false);
            MARGIN_LEFT = new CSS.Attribute("margin-left", "0", false);
            MARGIN_RIGHT = new CSS.Attribute("margin-right", "0", false);
            MARGIN_LEFT_LTR = new CSS.Attribute("margin-left-ltr", Integer.toString(Integer.MIN_VALUE), false);
            MARGIN_LEFT_RTL = new CSS.Attribute("margin-left-rtl", Integer.toString(Integer.MIN_VALUE), false);
            MARGIN_RIGHT_LTR = new CSS.Attribute("margin-right-ltr", Integer.toString(Integer.MIN_VALUE), false);
            MARGIN_RIGHT_RTL = new CSS.Attribute("margin-right-rtl", Integer.toString(Integer.MIN_VALUE), false);
            MARGIN_TOP = new CSS.Attribute("margin-top", "0", false);
            PADDING = new CSS.Attribute("padding", null, false);
            PADDING_BOTTOM = new CSS.Attribute("padding-bottom", "0", false);
            PADDING_LEFT = new CSS.Attribute("padding-left", "0", false);
            PADDING_RIGHT = new CSS.Attribute("padding-right", "0", false);
            PADDING_TOP = new CSS.Attribute("padding-top", "0", false);
            TEXT_ALIGN = new CSS.Attribute("text-align", null, true);
            TEXT_DECORATION = new CSS.Attribute("text-decoration", "none", true);
            TEXT_INDENT = new CSS.Attribute("text-indent", "0", true);
            TEXT_TRANSFORM = new CSS.Attribute("text-transform", "none", true);
            VERTICAL_ALIGN = new CSS.Attribute("vertical-align", "baseline", false);
            WORD_SPACING = new CSS.Attribute("word-spacing", "normal", true);
            WHITE_SPACE = new CSS.Attribute("white-space", "normal", true);
            WIDTH = new CSS.Attribute("width", "auto", false);
            BORDER_SPACING = new CSS.Attribute("border-spacing", "0", true);
            CAPTION_SIDE = new CSS.Attribute("caption-side", "left", true);
            allAttributes = new CSS.Attribute[] { CSS.Attribute.BACKGROUND, CSS.Attribute.BACKGROUND_ATTACHMENT, CSS.Attribute.BACKGROUND_COLOR, CSS.Attribute.BACKGROUND_IMAGE, CSS.Attribute.BACKGROUND_POSITION, CSS.Attribute.BACKGROUND_REPEAT, CSS.Attribute.BORDER, CSS.Attribute.BORDER_BOTTOM, CSS.Attribute.BORDER_BOTTOM_WIDTH, CSS.Attribute.BORDER_COLOR, CSS.Attribute.BORDER_LEFT, CSS.Attribute.BORDER_LEFT_WIDTH, CSS.Attribute.BORDER_RIGHT, CSS.Attribute.BORDER_RIGHT_WIDTH, CSS.Attribute.BORDER_STYLE, CSS.Attribute.BORDER_TOP, CSS.Attribute.BORDER_TOP_WIDTH, CSS.Attribute.BORDER_WIDTH, CSS.Attribute.BORDER_TOP_STYLE, CSS.Attribute.BORDER_RIGHT_STYLE, CSS.Attribute.BORDER_BOTTOM_STYLE, CSS.Attribute.BORDER_LEFT_STYLE, CSS.Attribute.BORDER_TOP_COLOR, CSS.Attribute.BORDER_RIGHT_COLOR, CSS.Attribute.BORDER_BOTTOM_COLOR, CSS.Attribute.BORDER_LEFT_COLOR, CSS.Attribute.CLEAR, CSS.Attribute.COLOR, CSS.Attribute.DISPLAY, CSS.Attribute.FLOAT, CSS.Attribute.FONT, CSS.Attribute.FONT_FAMILY, CSS.Attribute.FONT_SIZE, CSS.Attribute.FONT_STYLE, CSS.Attribute.FONT_VARIANT, CSS.Attribute.FONT_WEIGHT, CSS.Attribute.HEIGHT, CSS.Attribute.LETTER_SPACING, CSS.Attribute.LINE_HEIGHT, CSS.Attribute.LIST_STYLE, CSS.Attribute.LIST_STYLE_IMAGE, CSS.Attribute.LIST_STYLE_POSITION, CSS.Attribute.LIST_STYLE_TYPE, CSS.Attribute.MARGIN, CSS.Attribute.MARGIN_BOTTOM, CSS.Attribute.MARGIN_LEFT, CSS.Attribute.MARGIN_RIGHT, CSS.Attribute.MARGIN_TOP, CSS.Attribute.PADDING, CSS.Attribute.PADDING_BOTTOM, CSS.Attribute.PADDING_LEFT, CSS.Attribute.PADDING_RIGHT, CSS.Attribute.PADDING_TOP, CSS.Attribute.TEXT_ALIGN, CSS.Attribute.TEXT_DECORATION, CSS.Attribute.TEXT_INDENT, CSS.Attribute.TEXT_TRANSFORM, CSS.Attribute.VERTICAL_ALIGN, CSS.Attribute.WORD_SPACING, CSS.Attribute.WHITE_SPACE, CSS.Attribute.WIDTH, CSS.Attribute.BORDER_SPACING, CSS.Attribute.CAPTION_SIDE, CSS.Attribute.MARGIN_LEFT_LTR, CSS.Attribute.MARGIN_LEFT_RTL, CSS.Attribute.MARGIN_RIGHT_LTR, CSS.Attribute.MARGIN_RIGHT_RTL };
            ALL_MARGINS = new CSS.Attribute[] { CSS.Attribute.MARGIN_TOP, CSS.Attribute.MARGIN_RIGHT, CSS.Attribute.MARGIN_BOTTOM, CSS.Attribute.MARGIN_LEFT };
            ALL_PADDING = new CSS.Attribute[] { CSS.Attribute.PADDING_TOP, CSS.Attribute.PADDING_RIGHT, CSS.Attribute.PADDING_BOTTOM, CSS.Attribute.PADDING_LEFT };
            ALL_BORDER_WIDTHS = new CSS.Attribute[] { CSS.Attribute.BORDER_TOP_WIDTH, CSS.Attribute.BORDER_RIGHT_WIDTH, CSS.Attribute.BORDER_BOTTOM_WIDTH, CSS.Attribute.BORDER_LEFT_WIDTH };
            ALL_BORDER_STYLES = new CSS.Attribute[] { CSS.Attribute.BORDER_TOP_STYLE, CSS.Attribute.BORDER_RIGHT_STYLE, CSS.Attribute.BORDER_BOTTOM_STYLE, CSS.Attribute.BORDER_LEFT_STYLE };
            ALL_BORDER_COLORS = new CSS.Attribute[] { CSS.Attribute.BORDER_TOP_COLOR, CSS.Attribute.BORDER_RIGHT_COLOR, CSS.Attribute.BORDER_BOTTOM_COLOR, CSS.Attribute.BORDER_LEFT_COLOR };
        }
    }
    
    static final class Value
    {
        static final CSS.Value INHERITED;
        static final CSS.Value NONE;
        static final CSS.Value HIDDEN;
        static final CSS.Value DOTTED;
        static final CSS.Value DASHED;
        static final CSS.Value SOLID;
        static final CSS.Value DOUBLE;
        static final CSS.Value GROOVE;
        static final CSS.Value RIDGE;
        static final CSS.Value INSET;
        static final CSS.Value OUTSET;
        static final CSS.Value DISC;
        static final CSS.Value CIRCLE;
        static final CSS.Value SQUARE;
        static final CSS.Value DECIMAL;
        static final CSS.Value LOWER_ROMAN;
        static final CSS.Value UPPER_ROMAN;
        static final CSS.Value LOWER_ALPHA;
        static final CSS.Value UPPER_ALPHA;
        static final CSS.Value BACKGROUND_NO_REPEAT;
        static final CSS.Value BACKGROUND_REPEAT;
        static final CSS.Value BACKGROUND_REPEAT_X;
        static final CSS.Value BACKGROUND_REPEAT_Y;
        static final CSS.Value BACKGROUND_SCROLL;
        static final CSS.Value BACKGROUND_FIXED;
        private String name;
        static final CSS.Value[] allValues;
        
        private Value(String name) {
            this.name = name;
        }
        
        @Override
        public String toString() {
            return this.name;
        }
        
        static {
            INHERITED = new CSS.Value("inherited");
            NONE = new CSS.Value("none");
            HIDDEN = new CSS.Value("hidden");
            DOTTED = new CSS.Value("dotted");
            DASHED = new CSS.Value("dashed");
            SOLID = new CSS.Value("solid");
            DOUBLE = new CSS.Value("double");
            GROOVE = new CSS.Value("groove");
            RIDGE = new CSS.Value("ridge");
            INSET = new CSS.Value("inset");
            OUTSET = new CSS.Value("outset");
            DISC = new CSS.Value("disc");
            CIRCLE = new CSS.Value("circle");
            SQUARE = new CSS.Value("square");
            DECIMAL = new CSS.Value("decimal");
            LOWER_ROMAN = new CSS.Value("lower-roman");
            UPPER_ROMAN = new CSS.Value("upper-roman");
            LOWER_ALPHA = new CSS.Value("lower-alpha");
            UPPER_ALPHA = new CSS.Value("upper-alpha");
            BACKGROUND_NO_REPEAT = new CSS.Value("no-repeat");
            BACKGROUND_REPEAT = new CSS.Value("repeat");
            BACKGROUND_REPEAT_X = new CSS.Value("repeat-x");
            BACKGROUND_REPEAT_Y = new CSS.Value("repeat-y");
            BACKGROUND_SCROLL = new CSS.Value("scroll");
            BACKGROUND_FIXED = new CSS.Value("fixed");
            allValues = new CSS.Value[] { CSS.Value.INHERITED, CSS.Value.NONE, CSS.Value.DOTTED, CSS.Value.DASHED, CSS.Value.SOLID, CSS.Value.DOUBLE, CSS.Value.GROOVE, CSS.Value.RIDGE, CSS.Value.INSET, CSS.Value.OUTSET, CSS.Value.DISC, CSS.Value.CIRCLE, CSS.Value.SQUARE, CSS.Value.DECIMAL, CSS.Value.LOWER_ROMAN, CSS.Value.UPPER_ROMAN, CSS.Value.LOWER_ALPHA, CSS.Value.UPPER_ALPHA, CSS.Value.BACKGROUND_NO_REPEAT, CSS.Value.BACKGROUND_REPEAT, CSS.Value.BACKGROUND_REPEAT_X, CSS.Value.BACKGROUND_REPEAT_Y, CSS.Value.BACKGROUND_FIXED, CSS.Value.BACKGROUND_FIXED };
        }
    }
    
    static class CssValue implements Serializable
    {
        String svalue;
        
        Object parseCssValue(String s) {
            return s;
        }
        
        Object parseHtmlValue(String s) {
            return this.parseCssValue(s);
        }
        
        Object fromStyleConstants(StyleConstants styleConstants, Object o) {
            return null;
        }
        
        Object toStyleConstants(StyleConstants styleConstants, View view) {
            return null;
        }
        
        @Override
        public String toString() {
            return this.svalue;
        }
    }
    
    static class StringValue extends CSS.CssValue
    {
        @Override
        Object parseCssValue(String svalue) {
            CSS.StringValue stringValue = new CSS.StringValue();
            stringValue.svalue = svalue;
            return stringValue;
        }
        
        @Override
        Object fromStyleConstants(StyleConstants styleConstants, Object o) {
            if (styleConstants == StyleConstants.Italic) {
                if (o.equals(Boolean.TRUE)) {
                    return this.parseCssValue("italic");
                }
                return this.parseCssValue("");
            }
            else if (styleConstants == StyleConstants.Underline) {
                if (o.equals(Boolean.TRUE)) {
                    return this.parseCssValue("underline");
                }
                return this.parseCssValue("");
            }
            else {
                if (styleConstants == StyleConstants.Alignment) {
                    String s;
                    switch ((int)o) {
                        case 0: {
                            s = "left";
                            break;
                        }
                        case 2: {
                            s = "right";
                            break;
                        }
                        case 1: {
                            s = "center";
                            break;
                        }
                        case 3: {
                            s = "justify";
                            break;
                        }
                        default: {
                            s = "left";
                            break;
                        }
                    }
                    return this.parseCssValue(s);
                }
                if (styleConstants == StyleConstants.StrikeThrough) {
                    if (o.equals(Boolean.TRUE)) {
                        return this.parseCssValue("line-through");
                    }
                    return this.parseCssValue("");
                }
                else if (styleConstants == StyleConstants.Superscript) {
                    if (o.equals(Boolean.TRUE)) {
                        return this.parseCssValue("super");
                    }
                    return this.parseCssValue("");
                }
                else if (styleConstants == StyleConstants.Subscript) {
                    if (o.equals(Boolean.TRUE)) {
                        return this.parseCssValue("sub");
                    }
                    return this.parseCssValue("");
                }
                else {
                    return null;
                }
            }
        }
        
        @Override
        Object toStyleConstants(StyleConstants styleConstants, View view) {
            if (styleConstants == StyleConstants.Italic) {
                if (this.svalue.indexOf("italic") >= 0) {
                    return Boolean.TRUE;
                }
                return Boolean.FALSE;
            }
            else if (styleConstants == StyleConstants.Underline) {
                if (this.svalue.indexOf("underline") >= 0) {
                    return Boolean.TRUE;
                }
                return Boolean.FALSE;
            }
            else if (styleConstants == StyleConstants.Alignment) {
                if (this.svalue.equals("right")) {
                    return new Integer(2);
                }
                if (this.svalue.equals("center")) {
                    return new Integer(1);
                }
                if (this.svalue.equals("justify")) {
                    return new Integer(3);
                }
                return new Integer(0);
            }
            else if (styleConstants == StyleConstants.StrikeThrough) {
                if (this.svalue.indexOf("line-through") >= 0) {
                    return Boolean.TRUE;
                }
                return Boolean.FALSE;
            }
            else if (styleConstants == StyleConstants.Superscript) {
                if (this.svalue.indexOf("super") >= 0) {
                    return Boolean.TRUE;
                }
                return Boolean.FALSE;
            }
            else if (styleConstants == StyleConstants.Subscript) {
                if (this.svalue.indexOf("sub") >= 0) {
                    return Boolean.TRUE;
                }
                return Boolean.FALSE;
            }
            else {
                return null;
            }
        }
        
        boolean isItalic() {
            return this.svalue.indexOf("italic") != -1;
        }
        
        boolean isStrike() {
            return this.svalue.indexOf("line-through") != -1;
        }
        
        boolean isUnderline() {
            return this.svalue.indexOf("underline") != -1;
        }
        
        boolean isSub() {
            return this.svalue.indexOf("sub") != -1;
        }
        
        boolean isSup() {
            return this.svalue.indexOf("sup") != -1;
        }
    }
    
    class FontSize extends CSS.CssValue
    {
        float value;
        boolean index;
        CSS.LengthUnit lu;
        
        int getValue(AttributeSet set, StyleSheet access$500) {
            access$500 = CSS.access$500(CSS.this, access$500);
            if (this.index) {
                return Math.round(CSS.this.getPointSize((int)this.value, access$500));
            }
            if (this.lu == null) {
                return Math.round(this.value);
            }
            if (this.lu.type == 0) {
                return Math.round(this.lu.getValue(access$500 != null && access$500.isW3CLengthUnits()));
            }
            if (set != null) {
                AttributeSet resolveParent = set.getResolveParent();
                if (resolveParent != null) {
                    int fontSize = StyleConstants.getFontSize(resolveParent);
                    float a;
                    if (this.lu.type == 1 || this.lu.type == 3) {
                        a = this.lu.value * (float)fontSize;
                    }
                    else {
                        a = this.lu.value + (float)fontSize;
                    }
                    return Math.round(a);
                }
            }
            return 12;
        }
        
        @Override
        Object parseCssValue(String svalue) {
            CSS.FontSize fontSize = new CSS.FontSize();
            fontSize.svalue = svalue;
            try {
                if (svalue.equals("xx-small")) {
                    fontSize.value = 1.0f;
                    fontSize.index = true;
                }
                else if (svalue.equals("x-small")) {
                    fontSize.value = 2.0f;
                    fontSize.index = true;
                }
                else if (svalue.equals("small")) {
                    fontSize.value = 3.0f;
                    fontSize.index = true;
                }
                else if (svalue.equals("medium")) {
                    fontSize.value = 4.0f;
                    fontSize.index = true;
                }
                else if (svalue.equals("large")) {
                    fontSize.value = 5.0f;
                    fontSize.index = true;
                }
                else if (svalue.equals("x-large")) {
                    fontSize.value = 6.0f;
                    fontSize.index = true;
                }
                else if (svalue.equals("xx-large")) {
                    fontSize.value = 7.0f;
                    fontSize.index = true;
                }
                else {
                    fontSize.lu = new CSS.LengthUnit(svalue, (short)1, 1.0f);
                }
            }
            catch (NumberFormatException ex) {
                fontSize = null;
            }
            return fontSize;
        }
        
        @Override
        Object parseHtmlValue(String s) {
            if (s == null || s.length() == 0) {
                return null;
            }
            CSS.FontSize fontSize = new CSS.FontSize();
            fontSize.svalue = s;
            try {
                int baseFontSize = CSS.this.getBaseFontSize();
                if (s.charAt(0) == '+') {
                    fontSize.value = (float)(baseFontSize + (int)Integer.valueOf(s.substring(1)));
                    fontSize.index = true;
                }
                else if (s.charAt(0) == '-') {
                    fontSize.value = (float)(baseFontSize + -(int)Integer.valueOf(s.substring(1)));
                    fontSize.index = true;
                }
                else {
                    fontSize.value = (float)Integer.parseInt(s);
                    if (fontSize.value > 7.0f) {
                        fontSize.value = 7.0f;
                    }
                    else if (fontSize.value < 0.0f) {
                        fontSize.value = 0.0f;
                    }
                    fontSize.index = true;
                }
            }
            catch (NumberFormatException ex) {
                fontSize = null;
            }
            return fontSize;
        }
        
        @Override
        Object fromStyleConstants(StyleConstants styleConstants, Object o) {
            if ((o instanceof Number)) {
                CSS.FontSize fontSize = new CSS.FontSize();
                fontSize.value = (float)CSS.getIndexOfSize(((Number)o).floatValue(), StyleSheet.sizeMapDefault);
                fontSize.svalue = Integer.toString((int)fontSize.value);
                fontSize.index = true;
                return fontSize;
            }
            return this.parseCssValue(o.toString());
        }
        
        @Override
        Object toStyleConstants(StyleConstants styleConstants, View view) {
            if (view != null) {
                return Integer.valueOf(this.getValue(view.getAttributes(), null));
            }
            return Integer.valueOf(this.getValue(null, null));
        }
    }
    
    static class FontFamily extends CSS.CssValue
    {
        String family;
        
        String getValue() {
            return this.family;
        }
        
        @Override
        Object parseCssValue(String svalue) {
            int index = svalue.indexOf(44);
            CSS.FontFamily fontFamily = new CSS.FontFamily();
            fontFamily.svalue = svalue;
            fontFamily.family = null;
            if (index == -1) {
                this.setFontName(fontFamily, svalue);
            }
            else {
                int i = 0;
                int length = svalue.length();
                int index2 = 0;
                while (i == 0) {
                    while (index2 < length && Character.isWhitespace(svalue.charAt(index2))) {
                        ++index2;
                    }
                    index2 = svalue.indexOf(44, index2);
                    if (index2 == -1) {
                        index2 = length;
                    }
                    if (index2 < length) {
                        if (index2 != index2) {
                            if (index2 > 0 && svalue.charAt(index2 - 1) == ' ') {
                                --index2;
                            }
                            this.setFontName(fontFamily, svalue.substring(index2, index2));
                            i = (fontFamily.family != null);
                        }
                        ++index2;
                    }
                    else {
                        i = 1;
                    }
                }
            }
            if (fontFamily.family == null) {
                fontFamily.family = "SansSerif";
                return fontFamily;
            }
            return fontFamily;
        }
        
        private void setFontName(CSS.FontFamily fontFamily, String family) {
            fontFamily.family = family;
        }
        
        @Override
        Object parseHtmlValue(String s) {
            return this.parseCssValue(s);
        }
        
        @Override
        Object fromStyleConstants(StyleConstants styleConstants, Object o) {
            return this.parseCssValue(o.toString());
        }
        
        @Override
        Object toStyleConstants(StyleConstants styleConstants, View view) {
            return this.family;
        }
    }
    
    static class FontWeight extends CSS.CssValue
    {
        int weight;
        
        int getValue() {
            return this.weight;
        }
        
        @Override
        Object parseCssValue(String s) {
            CSS.FontWeight fontWeight = new CSS.FontWeight();
            fontWeight.svalue = s;
            if (s.equals("bold")) {
                fontWeight.weight = 700;
            }
            else if (s.equals("normal")) {
                fontWeight.weight = 400;
            }
            else {
                try {
                    fontWeight.weight = Integer.parseInt(s);
                }
                catch (NumberFormatException ex) {
                    fontWeight = null;
                }
            }
            return fontWeight;
        }
        
        @Override
        Object fromStyleConstants(StyleConstants styleConstants, Object o) {
            if (o.equals(Boolean.TRUE)) {
                return this.parseCssValue("bold");
            }
            return this.parseCssValue("normal");
        }
        
        @Override
        Object toStyleConstants(StyleConstants styleConstants, View view) {
            return (this.weight > 500) ? Boolean.TRUE : Boolean.FALSE;
        }
        
        boolean isBold() {
            return this.weight > 500;
        }
    }
    
    static class ColorValue extends CSS.CssValue
    {
        Color c;
        
        Color getValue() {
            return this.c;
        }
        
        @Override
        Object parseCssValue(String svalue) {
            Color stringToColor = CSS.stringToColor(svalue);
            if (stringToColor != null) {
                CSS.ColorValue colorValue = new CSS.ColorValue();
                colorValue.svalue = svalue;
                colorValue.c = stringToColor;
                return colorValue;
            }
            return null;
        }
        
        @Override
        Object parseHtmlValue(String s) {
            return this.parseCssValue(s);
        }
        
        @Override
        Object fromStyleConstants(StyleConstants styleConstants, Object o) {
            CSS.ColorValue colorValue = new CSS.ColorValue();
            colorValue.c = (Color)o;
            colorValue.svalue = CSS.colorToHex(colorValue.c);
            return colorValue;
        }
        
        @Override
        Object toStyleConstants(StyleConstants styleConstants, View view) {
            return this.c;
        }
    }
    
    static class BorderStyle extends CSS.CssValue
    {
        private transient CSS.Value style;
        
        CSS.Value getValue() {
            return this.style;
        }
        
        @Override
        Object parseCssValue(String svalue) {
            CSS.Value value = CSS.getValue(svalue);
            if (value != null && (value == CSS.Value.INSET || value == CSS.Value.OUTSET || value == CSS.Value.NONE || value == CSS.Value.DOTTED || value == CSS.Value.DASHED || value == CSS.Value.SOLID || value == CSS.Value.DOUBLE || value == CSS.Value.GROOVE || value == CSS.Value.RIDGE)) {
                CSS.BorderStyle borderStyle = new CSS.BorderStyle();
                borderStyle.svalue = svalue;
                borderStyle.style = value;
                return borderStyle;
            }
            return null;
        }
        
        private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
            objectOutputStream.defaultWriteObject();
            if (this.style == null) {
                objectOutputStream.writeObject(null);
            }
            else {
                objectOutputStream.writeObject(this.style.toString());
            }
        }
        
        private void readObject(ObjectInputStream objectInputStream) throws ClassNotFoundException, IOException {
            objectInputStream.defaultReadObject();
            Object object = objectInputStream.readObject();
            if (object != null) {
                this.style = CSS.getValue((String)object);
            }
        }
    }
    
    static class LengthValue extends CSS.CssValue
    {
        boolean mayBeNegative;
        boolean percentage;
        float span;
        String units;
        
        LengthValue() {
            this(false);
        }
        
        LengthValue(boolean mayBeNegative) {
            this.units = null;
            this.mayBeNegative = mayBeNegative;
        }
        
        float getValue() {
            return this.getValue(false);
        }
        
        float getValue(boolean b) {
            return this.getValue(0.0f, b);
        }
        
        float getValue(float n) {
            return this.getValue(n, false);
        }
        
        float getValue(float n, boolean b) {
            if (this.percentage) {
                return this.span * n;
            }
            return CSS.LengthUnit.getValue(this.span, this.units, Boolean.valueOf(b));
        }
        
        boolean isPercentage() {
            return this.percentage;
        }
        
        @Override
        Object parseCssValue(String s) {
            CSS.LengthValue lengthValue;
            try {
                float floatValue = (float)Float.valueOf(s);
                lengthValue = new CSS.LengthValue();
                lengthValue.span = floatValue;
            }
            catch (NumberFormatException ex) {
                CSS.LengthUnit lengthUnit = new CSS.LengthUnit(s, (short)10, 0.0f);
                switch (lengthUnit.type) {
                    case 0: {
                        lengthValue = new CSS.LengthValue();
                        lengthValue.span = (this.mayBeNegative ? lengthUnit.value : Math.max(0.0f, lengthUnit.value));
                        lengthValue.units = lengthUnit.units;
                        break;
                    }
                    case 1: {
                        lengthValue = new CSS.LengthValue();
                        lengthValue.span = Math.max(0.0f, Math.min(1.0f, lengthUnit.value));
                        lengthValue.percentage = true;
                        break;
                    }
                    default: {
                        return null;
                    }
                }
            }
            lengthValue.svalue = s;
            return lengthValue;
        }
        
        @Override
        Object parseHtmlValue(String s) {
            if (s.equals("#DEFAULT")) {
                s = "1";
            }
            return this.parseCssValue(s);
        }
        
        @Override
        Object fromStyleConstants(StyleConstants styleConstants, Object o) {
            CSS.LengthValue lengthValue = new CSS.LengthValue();
            lengthValue.svalue = o.toString();
            lengthValue.span = (float)o;
            return lengthValue;
        }
        
        @Override
        Object toStyleConstants(StyleConstants styleConstants, View view) {
            return new Float(this.getValue(false));
        }
    }
    
    static class BorderWidthValue extends CSS.LengthValue
    {
        private static final float[] values;
        
        BorderWidthValue(String svalue, int n) {
            this.svalue = svalue;
            this.span = CSS.BorderWidthValue.values[n];
            this.percentage = false;
        }
        
        @Override
        Object parseCssValue(String s) {
            if (s != null) {
                if (s.equals("thick")) {
                    return new CSS.BorderWidthValue(s, 2);
                }
                if (s.equals("medium")) {
                    return new CSS.BorderWidthValue(s, 1);
                }
                if (s.equals("thin")) {
                    return new CSS.BorderWidthValue(s, 0);
                }
            }
            return super.parseCssValue(s);
        }
        
        @Override
        Object parseHtmlValue(String s) {
            if (s == "#DEFAULT") {
                return this.parseCssValue("medium");
            }
            return this.parseCssValue(s);
        }
        
        static {
            values = new float[] { 1.0f, 2.0f, 4.0f };
        }
    }
    
    static class CssValueMapper extends CSS.CssValue
    {
        @Override
        Object parseCssValue(String key) {
            Object o = CSS.cssValueToInternalValueMap.get(key);
            if (o == null) {
                o = CSS.cssValueToInternalValueMap.get(key.toLowerCase());
                return o;
            }
            return o;
        }
        
        @Override
        Object parseHtmlValue(String key) {
            Object o = CSS.htmlValueToCssValueMap.get(key);
            if (o == null) {
                o = CSS.htmlValueToCssValueMap.get(key.toLowerCase());
                return o;
            }
            return o;
        }
    }
    
    static class BackgroundPosition extends CSS.CssValue
    {
        float horizontalPosition;
        float verticalPosition;
        short relative;
        
        @Override
        Object parseCssValue(String svalue) {
            String[] strings = CSS.parseStrings(svalue);
            int length = strings.length;
            CSS.BackgroundPosition backgroundPosition = new CSS.BackgroundPosition();
            backgroundPosition.relative = 5;
            backgroundPosition.svalue = svalue;
            if (length > 0) {
                int n = 0;
                int i = 0;
                while (i < length) {
                    String s = strings[i++];
                    if (s.equals("center")) {
                        n = (short)(n | 0x4);
                    }
                    else {
                        if ((n & 0x1) == 0x0) {
                            if (s.equals("top")) {
                                n = (short)(n | 0x1);
                            }
                            else if (s.equals("bottom")) {
                                n = (short)(n | 0x1);
                                backgroundPosition.verticalPosition = 1.0f;
                                continue;
                            }
                        }
                        if ((n & 0x2) == 0x0) {
                            if (s.equals("left")) {
                                n = (short)(n | 0x2);
                                backgroundPosition.horizontalPosition = 0.0f;
                            }
                            else if (s.equals("right")) {
                                n = (short)(n | 0x2);
                                backgroundPosition.horizontalPosition = 1.0f;
                            }
                            else {
                                continue;
                            }
                        }
                        else {
                            continue;
                        }
                    }
                }
                if (n != 0) {
                    if ((n & 0x1) == 0x1) {
                        if ((n & 0x2) == 0x0) {
                            backgroundPosition.horizontalPosition = 0.5f;
                        }
                    }
                    else if ((n & 0x2) == 0x2) {
                        backgroundPosition.verticalPosition = 0.5f;
                    }
                    else {
                        CSS.BackgroundPosition backgroundPosition2 = backgroundPosition;
                        float n2 = 0.5f;
                        backgroundPosition.verticalPosition = n2;
                        backgroundPosition2.horizontalPosition = n2;
                    }
                }
                else {
                    CSS.LengthUnit o = new CSS.LengthUnit(strings[0], (short)0, 0.0f);
                    if (o.type == 0) {
                        backgroundPosition.horizontalPosition = o.value;
                        backgroundPosition.relative ^= 0x1;
                    }
                    else if (o.type == 1) {
                        backgroundPosition.horizontalPosition = o.value;
                    }
                    else if (o.type == 3) {
                        backgroundPosition.horizontalPosition = o.value;
                        backgroundPosition.relative = (short)((0x1 ^ backgroundPosition.relative) | 0x2);
                    }
                    if (length > 1) {
                        CSS.LengthUnit o2 = new CSS.LengthUnit(strings[1], (short)0, 0.0f);
                        if (o2.type == 0) {
                            backgroundPosition.verticalPosition = o2.value;
                            backgroundPosition.relative ^= 0x4;
                        }
                        else if (o2.type == 1) {
                            backgroundPosition.verticalPosition = o2.value;
                        }
                        else if (o2.type == 3) {
                            backgroundPosition.verticalPosition = o2.value;
                            backgroundPosition.relative = (short)((0x4 ^ backgroundPosition.relative) | 0x8);
                        }
                    }
                    else {
                        backgroundPosition.verticalPosition = 0.5f;
                    }
                }
            }
            return backgroundPosition;
        }
        
        boolean isHorizontalPositionRelativeToSize() {
            return (this.relative & 0x1) == 0x1;
        }
        
        boolean isHorizontalPositionRelativeToFontSize() {
            return (this.relative & 0x2) == 0x2;
        }
        
        float getHorizontalPosition() {
            return this.horizontalPosition;
        }
        
        boolean isVerticalPositionRelativeToSize() {
            return (this.relative & 0x4) == 0x4;
        }
        
        boolean isVerticalPositionRelativeToFontSize() {
            return (this.relative & 0x8) == 0x8;
        }
        
        float getVerticalPosition() {
            return this.verticalPosition;
        }
    }
    
    static class BackgroundImage extends CSS.CssValue
    {
        private boolean loadedImage;
        private ImageIcon image;
        
        @Override
        Object parseCssValue(String svalue) {
            CSS.BackgroundImage backgroundImage = new CSS.BackgroundImage();
            backgroundImage.svalue = svalue;
            return backgroundImage;
        }
        
        @Override
        Object parseHtmlValue(String s) {
            return this.parseCssValue(s);
        }
        
        ImageIcon getImage(URL url) {
            if (!this.loadedImage) {
                synchronized (this) {
                    if (!this.loadedImage) {
                        URL url2 = CSS.getURL(url, this.svalue);
                        this.loadedImage = true;
                        if (url2 != null) {
                            this.image = new ImageIcon();
                            Image image = Toolkit.getDefaultToolkit().createImage(url2);
                            if (image != null) {
                                this.image.setImage(image);
                            }
                        }
                    }
                }
            }
            return this.image;
        }
    }
    
    static class LengthUnit implements Serializable
    {
        static Hashtable<String, Float> lengthMapping;
        static Hashtable<String, Float> w3cLengthMapping;
        short type;
        float value;
        String units;
        static final short UNINITALIZED_LENGTH = 10;
        
        LengthUnit(String s, short n, float n2) {
            this.units = null;
            this.parse(s, n, n2);
        }
        
        void parse(String s, short type, float value) {
            this.type = type;
            this.value = value;
            int length = s.length();
            if (length > 0 && s.charAt(length - 1) == '%') {
                try {
                    this.value = (float)Float.valueOf(s.substring(0, length - 1)) / 100.0f;
                    this.type = 1;
                }
                catch (NumberFormatException ex) {}
            }
            if (length >= 2) {
                this.units = s.substring(length - 2, length);
                if ((Float)CSS.LengthUnit.lengthMapping.get(this.units) != null) {
                    try {
                        this.value = (float)Float.valueOf(s.substring(0, length - 2));
                        this.type = 0;
                    }
                    catch (NumberFormatException ex5) {}
                }
                else {
                    if (!this.units.equals("em") && !this.units.equals("ex")) {
                        if (s.equals("larger")) {
                            this.value = 2.0f;
                            this.type = 2;
                            return;
                        }
                        if (s.equals("smaller")) {
                            this.value = -2.0f;
                            this.type = 2;
                            return;
                        }
                        try {
                            this.value = (float)Float.valueOf(s);
                            this.type = 0;
                        }
                        catch (NumberFormatException ex3) {}
                        return;
                    }
                    try {
                        this.value = (float)Float.valueOf(s.substring(0, length - 2));
                        this.type = 3;
                    }
                    catch (NumberFormatException ex4) {}
                }
            }
            else if (length > 0) {
                try {
                    this.value = (float)Float.valueOf(s);
                    this.type = 0;
                }
                catch (NumberFormatException ex2) {}
            }
        }
        
        float getValue(boolean b) {
            Hashtable<Object, Object> hashtable = b ? CSS.LengthUnit.w3cLengthMapping : CSS.LengthUnit.lengthMapping;
            float floatValue = 1.0f;
            if (this.units != null) {
                Float n = (Float)hashtable.get(this.units);
                if (n != null) {
                    floatValue = (float)n;
                }
            }
            return this.value * floatValue;
        }
        
        static float getValue(float n, String key, Boolean b) {
            Hashtable<Object, Object> hashtable = ((boolean)b) ? CSS.LengthUnit.w3cLengthMapping : CSS.LengthUnit.lengthMapping;
            float floatValue = 1.0f;
            if (key != null) {
                Float n2 = (Float)hashtable.get(key);
                if (n2 != null) {
                    floatValue = (float)n2;
                }
            }
            return n * floatValue;
        }
        
        @Override
        public String toString() {
            return (int)this.type + " " + this.value;
        }
        
        static {
            CSS.LengthUnit.lengthMapping = new Hashtable(6);
            CSS.LengthUnit.w3cLengthMapping = new Hashtable(6);
            CSS.LengthUnit.lengthMapping.put("pt", new Float(1.0f));
            CSS.LengthUnit.lengthMapping.put("px", new Float(1.3f));
            CSS.LengthUnit.lengthMapping.put("mm", new Float(2.83464f));
            CSS.LengthUnit.lengthMapping.put("cm", new Float(28.3464f));
            CSS.LengthUnit.lengthMapping.put("pc", new Float(12.0f));
            CSS.LengthUnit.lengthMapping.put("in", new Float(72.0f));
            int screenResolution = 72;
            try {
                screenResolution = Toolkit.getDefaultToolkit().getScreenResolution();
            }
            catch (HeadlessException ex) {}
            CSS.LengthUnit.w3cLengthMapping.put("pt", new Float((float)screenResolution / 72.0f));
            CSS.LengthUnit.w3cLengthMapping.put("px", new Float(1.0f));
            CSS.LengthUnit.w3cLengthMapping.put("mm", new Float((float)screenResolution / 25.4f));
            CSS.LengthUnit.w3cLengthMapping.put("cm", new Float((float)screenResolution / 2.54f));
            CSS.LengthUnit.w3cLengthMapping.put("pc", new Float((float)screenResolution / 6.0f));
            CSS.LengthUnit.w3cLengthMapping.put("in", new Float((float)screenResolution));
        }
    }
    
    static class ShorthandFontParser
    {
        static void parseShorthandFont(CSS css, String s, MutableAttributeSet set) {
            String[] strings = CSS.parseStrings(s);
            int length = strings.length;
            int i = 0;
            int n = 0;
            while (i < Math.min(3, length)) {
                if ((n & 0x1) == 0x0 && isFontStyle(strings[i])) {
                    css.addInternalCSSValue(set, CSS.Attribute.FONT_STYLE, strings[i++]);
                    n = (short)(n | 0x1);
                }
                else if ((n & 0x2) == 0x0 && isFontVariant(strings[i])) {
                    css.addInternalCSSValue(set, CSS.Attribute.FONT_VARIANT, strings[i++]);
                    n = (short)(n | 0x2);
                }
                else if ((n & 0x4) == 0x0 && isFontWeight(strings[i])) {
                    css.addInternalCSSValue(set, CSS.Attribute.FONT_WEIGHT, strings[i++]);
                    n = (short)(n | 0x4);
                }
                else if (strings[i].equals("normal")) {
                    ++i;
                }
                else {
                    break;
                }
            }
            if ((n & 0x1) == 0x0) {
                css.addInternalCSSValue(set, CSS.Attribute.FONT_STYLE, "normal");
            }
            if ((n & 0x2) == 0x0) {
                css.addInternalCSSValue(set, CSS.Attribute.FONT_VARIANT, "normal");
            }
            if ((n & 0x4) == 0x0) {
                css.addInternalCSSValue(set, CSS.Attribute.FONT_WEIGHT, "normal");
            }
            if (i < length) {
                String substring = strings[i];
                int index = substring.indexOf(47);
                if (index != -1) {
                    substring = substring.substring(0, index);
                    strings[i] = strings[i].substring(index);
                }
                else {
                    ++i;
                }
                css.addInternalCSSValue(set, CSS.Attribute.FONT_SIZE, substring);
            }
            else {
                css.addInternalCSSValue(set, CSS.Attribute.FONT_SIZE, "medium");
            }
            if (i < length && strings[i].startsWith("/")) {
                String substring2 = null;
                if (strings[i].equals("/")) {
                    if (++i < length) {
                        substring2 = strings[i++];
                    }
                }
                else {
                    substring2 = strings[i++].substring(1);
                }
                if (substring2 != null) {
                    css.addInternalCSSValue(set, CSS.Attribute.LINE_HEIGHT, substring2);
                }
                else {
                    css.addInternalCSSValue(set, CSS.Attribute.LINE_HEIGHT, "normal");
                }
            }
            else {
                css.addInternalCSSValue(set, CSS.Attribute.LINE_HEIGHT, "normal");
            }
            if (i < length) {
                String string;
                for (string = strings[i++]; i < length; string = string + " " + strings[i++]) {}
                css.addInternalCSSValue(set, CSS.Attribute.FONT_FAMILY, string);
            }
            else {
                css.addInternalCSSValue(set, CSS.Attribute.FONT_FAMILY, "SansSerif");
            }
        }
        
        private static boolean isFontStyle(String s) {
            return s.equals("italic") || s.equals("oblique");
        }
        
        private static boolean isFontVariant(String s) {
            return s.equals("small-caps");
        }
        
        private static boolean isFontWeight(String s) {
            return s.equals("bold") || s.equals("bolder") || s.equals("italic") || s.equals("lighter") || (s.length() == 3 && s.charAt(0) >= '1' && s.charAt(0) <= '9' && s.charAt(1) == '0' && s.charAt(2) == '0');
        }
    }
    
    static class ShorthandBackgroundParser
    {
        static void parseShorthandBackground(CSS css, String s, MutableAttributeSet set) {
            String[] strings = CSS.parseStrings(s);
            int length = strings.length;
            int i = 0;
            int n = 0;
            while (i < length) {
                String str = strings[i++];
                if ((n & 0x1) == 0x0 && isImage(str)) {
                    css.addInternalCSSValue(set, CSS.Attribute.BACKGROUND_IMAGE, str);
                    n = (short)(n | 0x1);
                }
                else if ((n & 0x2) == 0x0 && isRepeat(str)) {
                    css.addInternalCSSValue(set, CSS.Attribute.BACKGROUND_REPEAT, str);
                    n = (short)(n | 0x2);
                }
                else if ((n & 0x4) == 0x0 && isAttachment(str)) {
                    css.addInternalCSSValue(set, CSS.Attribute.BACKGROUND_ATTACHMENT, str);
                    n = (short)(n | 0x4);
                }
                else if ((n & 0x8) == 0x0 && isPosition(str)) {
                    if (i < length && isPosition(strings[i])) {
                        css.addInternalCSSValue(set, CSS.Attribute.BACKGROUND_POSITION, str + " " + strings[i++]);
                    }
                    else {
                        css.addInternalCSSValue(set, CSS.Attribute.BACKGROUND_POSITION, str);
                    }
                    n = (short)(n | 0x8);
                }
                else if ((n & 0x10) == 0x0 && isColor(str)) {
                    css.addInternalCSSValue(set, CSS.Attribute.BACKGROUND_COLOR, str);
                    n = (short)(n | 0x10);
                }
                else {
                    continue;
                }
            }
            if ((n & 0x1) == 0x0) {
                css.addInternalCSSValue(set, CSS.Attribute.BACKGROUND_IMAGE, null);
            }
            if ((n & 0x2) == 0x0) {
                css.addInternalCSSValue(set, CSS.Attribute.BACKGROUND_REPEAT, "repeat");
            }
            if ((n & 0x4) == 0x0) {
                css.addInternalCSSValue(set, CSS.Attribute.BACKGROUND_ATTACHMENT, "scroll");
            }
            if ((n & 0x8) == 0x0) {
                css.addInternalCSSValue(set, CSS.Attribute.BACKGROUND_POSITION, null);
            }
        }
        
        static boolean isImage(String s) {
            return s.startsWith("url(") && s.endsWith(")");
        }
        
        static boolean isRepeat(String s) {
            return s.equals("repeat-x") || s.equals("repeat-y") || s.equals("repeat") || s.equals("no-repeat");
        }
        
        static boolean isAttachment(String s) {
            return s.equals("fixed") || s.equals("scroll");
        }
        
        static boolean isPosition(String s) {
            return s.equals("top") || s.equals("bottom") || s.equals("left") || s.equals("right") || s.equals("center") || (s.length() > 0 && Character.isDigit(s.charAt(0)));
        }
        
        static boolean isColor(String s) {
            return CSS.stringToColor(s) != null;
        }
    }
    
    static class ShorthandMarginParser
    {
        static void parseShorthandMargin(CSS css, String s, MutableAttributeSet set, CSS.Attribute[] array) {
            String[] strings = CSS.parseStrings(s);
            switch (strings.length) {
                case 0: {
                    return;
                }
                case 1: {
                    for (int i = 0; i < 4; ++i) {
                        css.addInternalCSSValue(set, array[i], strings[0]);
                    }
                    break;
                }
                case 2: {
                    css.addInternalCSSValue(set, array[0], strings[0]);
                    css.addInternalCSSValue(set, array[2], strings[0]);
                    css.addInternalCSSValue(set, array[1], strings[1]);
                    css.addInternalCSSValue(set, array[3], strings[1]);
                    break;
                }
                case 3: {
                    css.addInternalCSSValue(set, array[0], strings[0]);
                    css.addInternalCSSValue(set, array[1], strings[1]);
                    css.addInternalCSSValue(set, array[2], strings[2]);
                    css.addInternalCSSValue(set, array[3], strings[1]);
                    break;
                }
                default: {
                    for (int j = 0; j < 4; ++j) {
                        css.addInternalCSSValue(set, array[j], strings[j]);
                    }
                    break;
                }
            }
        }
    }
    
    static class ShorthandBorderParser
    {
        static CSS.Attribute[] keys;
        
        static void parseShorthandBorder(MutableAttributeSet set, CSS.Attribute attribute, String s) {
            Object[] array = new Object[CSSBorder.PARSERS.length];
            String[] strings = CSS.parseStrings(s);
            int length = strings.length;
            int i = 0;
            while (i < length) {
                String s2 = strings[i];
                boolean n = false;
                int j = 0;
                while (j < array.length) {
                    Object cssValue = CSSBorder.PARSERS[j].parseCssValue(s2);
                    if (cssValue != null) {
                        if (array[j] == null) {
                            array[j] = cssValue;
                            n = true;
                            break;
                        }
                        break;
                    }
                    else {
                        ++j;
                    }
                }
                if (n == 0) {
                    return;
                }
                ++i;
            }
            for (int k = 0; k < array.length; ++k) {
                if (array[k] == null) {
                    array[k] = CSSBorder.DEFAULTS[k];
                }
            }
            for (int l = 0; l < CSS.ShorthandBorderParser.keys.length; ++l) {
                if (attribute == CSS.Attribute.BORDER || attribute == CSS.ShorthandBorderParser.keys[l]) {
                    for (int n2 = 0; n2 < array.length; ++n2) {
                        set.addAttribute(CSSBorder.ATTRIBUTES[n2][l], array[n2]);
                    }
                }
            }
        }
        
        static {
            CSS.ShorthandBorderParser.keys = new CSS.Attribute[] { CSS.Attribute.BORDER_TOP, CSS.Attribute.BORDER_RIGHT, CSS.Attribute.BORDER_BOTTOM, CSS.Attribute.BORDER_LEFT };
        }
    }
    
    interface LayoutIterator
    {
        public static final int WorstAdjustmentWeight = 2;
        
        public abstract void setOffset(int p0);
        
        public abstract int getOffset();
        
        public abstract void setSpan(int p0);
        
        public abstract int getSpan();
        
        public abstract int getCount();
        
        public abstract void setIndex(int p0);
        
        public abstract float getMinimumSpan(float p0);
        
        public abstract float getPreferredSpan(float p0);
        
        public abstract float getMaximumSpan(float p0);
        
        public abstract int getAdjustmentWeight();
        
        public abstract float getBorderWidth();
        
        public abstract float getLeadingCollapseSpan();
        
        public abstract float getTrailingCollapseSpan();
    }
}
